/*
 * Copyright (c) 2002 Ian Dowse.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $FreeBSD$
 */

/*
 * Realtek RTL8150(M) USB ethernet controller definitions.
 */

/*
 * Access to registers (UT_*_VENDOR_DEVICE)
 */
#define RUE_CMD_WRITEREG			0x05
#define RUE_CMD_READREG				0x05

/*
 * Device registers
 */
#define RUE_IDR			0x120	/* Ethernet address (6 bytes) */
#define RUE_MAR			0x126	/* Multicast (8 bytes) */
#define RUE_CR			0x12e	/* Command register */
#define RUE_TCR			0x12f	/* TX config register */
#define RUE_RCR			0x130	/* RX config register (16-bit) */
#define RUE_TSR			0x132	/* TX status register */
#define RUE_RSR			0x133	/* RX status register */
/* 0x134 Reserved */
#define RUE_CON0		0x135	/* Config register 0 */
#define RUE_CON1		0x136	/* Config register 1 */
#define RUE_MSR			0x137	/* Media status register */
#define RUE_PHYADD		0x138	/* MII PHY address select */
#define RUE_PHYDAT		0x139	/* MII PHY data (16-bit) */
#define RUE_PHYCNT		0x13b	/* MII PHY control */
/* 0x13c Reserved */
#define RUE_GPPC		0x13d	/* General purpose pin control reg */
#define RUE_WAKECNT		0x13e	/* Wakeup event control */
#define RUE_BMCR		0x140	/* Basic mode ctrl reg (16-bit) */
#define RUE_BMSR		0x142	/* Basic mode status reg (16-bit) */
#define RUE_ANAR		0x144	/* Autoneg advertisement (16-bit) */
#define RUE_ANLP		0x146	/* Autoneg link partner ability (16) */
#define RUE_AER			0x148	/* Autoneg expansion register (16) */
#define RUE_NWAYT		0x14a	/* Nway test register (16-bit) */
#define RUE_CSCR		0x14c	/* CS config register (16-bit) */
#define RUE_CRC			0x14e	/* PM CRC for wakeup frame (5*16-bit) */
#define RUE_BYTEMASK		0x158	/* PM wakeup frame (5*64-bit) */
#define RUE_PHY1		0x180	/* PHY parameter 1 (32-bit) */
#define RUE_PHY2		0x184	/* PHY parameter 2 */
#define RUE_TW			0x186	/* Twisted parameter 1 (32-bit) */

/* Command register (RUE_CR) */
#define RUE_CR_AUTOLOAD		0x01	/* Auto-load eeprom to registers */
#define RUE_CR_EP3CLEAN		0x02	/* Auto-clean intr stats on read */
#define RUE_CR_TE		0x04	/* Ethernet transmit enable */
#define RUE_CR_RE		0x08	/* Ethernet receive enable */
#define RUE_CR_SOFTRESET	0x10	/* Software reset enable */
#define RUE_CR_WEPROM		0x20	/* Write-enable eeprom */

/* TX config register (RUE_TCR) */
#define RUE_TCR_NOCRC		0x01	/* Don't append CRC to packet */
#define RUE_TCR_IFG0		0x02	/* Interframe gap control 0 */
#define RUE_TCR_IFG1		0x04	/* Interframe gap control 1 */
#define RUE_TCR_TRR0		0x40	/* TX retry count 0 */
#define RUE_TCR_TRR1		0x80	/* TX retry count 1 */

/* RX config register (RUE_RCR) (16-bit) */
#define RUE_RCR_AAP		0x0001	/* Accept all physical frames */
#define RUE_RCR_AAM		0x0002	/* Accept all multicast frames */
#define RUE_RCR_AD		0x0004	/* Accept if dest MAC ours */
#define RUE_RCR_AB		0x0008	/* Accept broadcasts */
#define RUE_RCR_AM		0x0010	/* Accept matched multicast frames */
#define RUE_RCR_AR		0x0020	/* Accept runt packets */
#define RUE_RCR_AER		0x0040	/* Accept packets with CRC errors */
#define RUE_RCR_TAIL		0x0080	/* Don't forward CRC to host */

/* TX status register (RUE_TSR) */
#define RUE_TSR_TXFULL		0x01	/* TX buffer full */
#define RUE_TSR_TXEMPTY		0x02	/* TX buffer empty */
#define RUE_TSR_JBR		0x04	/* Jabber timeout */
#define RUE_TSR_NOCARRIER	0x08	/* No carrier */
#define RUE_TSR_LCOL		0x10	/* Late collision */
#define RUE_TSR_ECOL		0x20	/* Excessive collisions */

/* RX status register (RUE_RSR) */
#define RUE_RSR_ROK		0x01	/* RX OK */
#define RUE_RSR_FAE		0x02	/* Frame alignment error */
#define RUE_RSR_CRC		0x04	/* CRC error */
#define RUE_RSR_LONG		0x08	/* Long packet */
#define RUE_RSR_RUNT		0x10	/* Runt packet */
#define RUE_RSR_LKCHG		0x20	/* Link change */
#define RUE_RSR_RXFULL		0x40	/* RX buffer full */
#define RUE_RSR_WEVENT		0x80	/* Wakeup event */

/* Config register 0 (RUE_CON0) */
#define RUE_CON0_LEDS0		0x01	/* LED assignment control 0 */
#define RUE_CON0_LEDS1		0x02	/* LED assignment control 1 */
#define RUE_CON0_MSEL		0x04	/* MII mode (disable internal PHY) */
#define RUE_CON0_LDPS		0x08	/* Disable link down power saving */
#define RUE_CON0_PARMEN		0x40	/* Permit modification of PHY0-1,TW */
#define RUE_CON0_SUSLED		0x80	/* Don't disable LEDs in suspend */

/* Config register 1 (RUE_CON1) */
#define RUE_CON0_LONGWF0	0x01	/* Join BYTEMASK1/2 for w/up frame 0 */
#define RUE_CON0_LONGWF1	0x02	/* Join BYTEMASK3/4 for w/up frame 1 */
#define RUE_CON0_UWF		0x10	/* Enable unicast wakeup frame */
#define RUE_CON0_MWF		0x20	/* Enable multicast wakeup frame */
#define RUE_CON0_BWF		0x40	/* Enable broadcast wakeup frame */

/* Media status register (RUE_MSR) */
#define RUE_MSR_RXPF		0x01	/* Received pause packet */
#define RUE_MSR_TXPF		0x02	/* Sent pause packet */
#define RUE_MSR_LINK		0x04	/* Link status OK */
#define RUE_MSR_SPEED100	0x08	/* Speed is 100Mbps */
#define RUE_MSR_DUPLEX		0x10	/* Link is full-duplex */
#define RUE_MSR_RXFCE		0x40	/* Enable RX flow-control */
#define RUE_MSR_TXFCE		0x80	/* Enable TX flow-control */

/* General purpose pin control reg (RUE_GPPC) */
#define RUE_GPPC_GEP0RW		0x01	/* Pin GEP0 is output */
#define RUE_GPPC_GEP0DAT	0x02	/* Read/write pin GEP0 */
#define RUE_GPPC_GEP1RW		0x04	/* Pin GEP1 is output */
#define RUE_GPPC_GEP1DAT	0x08	/* Read/write pin GEP1 */
#define RUE_GPPC_GEPREG0	0x10	/* General purpose bit */

/* Wakeup event control (RUE_WAKECNT) */
#define RUE_WAKECNT_WUF0EN	0x01	/* Wakeup frame 0 enable */
#define RUE_WAKECNT_WUF1EN	0x02	/* Wakeup frame 1 enable */
#define RUE_WAKECNT_WUF2EN	0x04	/* Wakeup frame 2 enable */
#define RUE_WAKECNT_WUF3EN	0x08	/* Wakeup frame 3 enable */
#define RUE_WAKECNT_WUF4EN	0x10	/* Wakeup frame 4 enable */
#define RUE_WAKECNT_MAGWEN	0x20	/* Magic packet wakeup enable */
#define RUE_WAKECNT_LKWEN	0x40	/* Link change wakeup enable */

/* Basic mode ctrl reg (16-bit) */
#define RUE_BMCR_FULLDUPLEX	0x0100	/* Full duplex mode */
#define RUE_BMCR_INITNEG	0x0200	/* Restart auto negotiation */
#define RUE_BMCR_ANE		0x1000	/* Auto negotiation enable */
#define RUE_BMCR_SPEED100	0x2000	/* Set 100Mbps speed */
#define RUE_BMCR_RESET		0x8000	/* Reset PHY control and status regs */

/* Basic mode status reg (16-bit) */
#define RUE_BMSR_EXTENDED	0x0001	/* Extended register capabilities */
#define RUE_BMSR_AUTONEG	0x0008	/* Device supports auto-negotiation */
#define RUE_BMSR_RFAULT		0x0010	/* Remote fault */
#define RUE_BMSR_NEGDONE	0x0020	/* Auto-negotiation completed */
#define RUE_BMSR_10BASET_HD	0x0800	/* 10baseT half-duplex supported */
#define RUE_BMSR_10BASET_FD	0x1000	/* 10baseT full-duplex supported */
#define RUE_BMSR_100BASETX_HD	0x2000	/* 100baseTX half-duplex supported */
#define RUE_BMSR_100BASETX_FD	0x4000	/* 100baseTX full-duplex supported */
#define RUE_BMSR_100BASET4	0x8000	/* 100baseT4 supported */

/* Autoneg advertisement (16-bit) */
#define RUE_ANAR_10BASET_HD	0x0020	/* Advertise 10baseT half-duplex */
#define RUE_ANAR_10BASET_FD	0x0040	/* Advertise 10baseT full-duplex */
#define RUE_ANAR_100BASETX_HD	0x0080	/* Advertise 100baseTX half-duplex */
#define RUE_ANAR_100BASETX_FD	0x0100	/* Advertise 100baseTX full-duplex */
#define RUE_ANAR_100BASET4	0x0200	/* Advertise 100baseT4 */
#define RUE_ANAR_PAUSE		0x0400	/* Advertise flow-control */
#define RUE_ANAR_RFAULT		0x2000	/* Advertise remote fault */
#define RUE_ANAR_ACK		0x4000	/* Ack link partner's cap word */
#define RUE_ANAR_NP		0x8000	/* Advertise next-page capability */

/* Autoneg link partner ability (16-bit) */
/* Autoneg expansion register (16-bit) */
/* Nway test register (16-bit) */

/* CS config register (16-bit) */
#define RUE_CS_PASSSCR		0x0001	/* Bypass scrambler */
#define RUE_CS_ENSTATUS		0x0004	/* Force connection status LED */
#define RUE_CS_STATUS		0x0008	/* (RO) Connection status */
#define RUE_CS_CONNECT		0x0020	/* Force connection of link */
#define RUE_CS_LINK100		0x0040	/* Force 100Mbps link-up */
#define RUE_CS_JBEN		0x0080	/* Enable jabber */
#define RUE_CS_HEARTBEAT	0x0100	/* Enable heart beat (10Mbps/s) */
#define RUE_CS_LD		0x0200	/* Active-low TP1 link disable signal */
#define RUE_CS_TESTFUN		0x8000	/* Speed up autoneg timer */

/* RX header (16-bit) */
#define RUE_RXHDR_BYTECNTMASK	0x0fff	/* Count of bytes in packet */
#define RUE_RXHDR_VALID		0x1000	/* Valid packet */
#define RUE_RXHDR_RUNT		0x2000	/* Runt packet */
#define RUE_RXHDR_PHYSMATCH	0x4000	/* Physical match */
#define RUE_RXHDR_MCASTMATCH	0x8000	/* Multicast match */

#define RUE_ENDPT_RX		0x0
#define RUE_ENDPT_TX		0x1
#define RUE_ENDPT_INTR		0x2
#define RUE_ENDPT_MAX		0x3

struct rue_intrdata {
	u_char rue_tsr;			/* Same as RUE_TSR */
	u_char rue_rsr;			/* Same as RUE_RSR */
	u_char rue_gepmsr;		/* Bits 0-4 of RUE_MSR + GEP0/1 */
	u_char rue_waksr;
	u_char rue_txok_cnt;		/* Valid packets transmitted */
	u_char rue_rxlost_cnt;		/* Lost packets due to RX buffer full */
	u_char rue_crcerr_cnt;		/* Packets with CRC errors */
	u_char rue_col_cnt;		/* Collisions */
};
